package to.mambo;

import netscape.application.Application;
import netscape.application.Target;
import netscape.application.TextView;
import netscape.application.TextViewOwner;
import netscape.application.TextFilter;
import netscape.application.KeyEvent;
import netscape.application.Range;

import netscape.util.Vector;

import java.io.IOException;
import java.io.Reader;
import java.io.Writer;
//import java.io.PipedReader;
// import java.io.PipedWriter;

import to.mambo.PipedReader;
import to.mambo.PipedWriter;

public class Console
{
   ConsoleViewOwner  viewOwner;

   final PipedReader    inPipe  = new PipedReader();
   final PipedWriter    outPipe = new PipedWriter();

   final public Reader in  = inPipe;
   final public Writer out = outPipe;
   final public Writer err = outPipe;

   public Console(TextView v) throws IOException
   {
      viewOwner = new ConsoleViewOwner(v);

      // The user's output goes to our console input.
      outPipe.connect(viewOwner.readPipe);

      // The user's input comes from our console output.
      inPipe.connect(viewOwner.writePipe);
   }

   public void enable()
   {
      // writer.start();
      viewOwner.reader.start();
   }

}

class ConsoleViewOwner implements TextViewOwner, TextFilter, Target
{
   TextView       textView;

   CharQueue      charQueue = new CharQueue();

   PipedReader    readPipe  = new PipedReader();
   PipedWriter    writePipe = new PipedWriter();

   Thread         reader = new Thread(new ConsoleReader(this));
   Thread         writer = new Thread(new ConsoleWriter(this));

   boolean        failure = false;

   boolean        selectionAtEnd = false;
   boolean        justSawReturn = false;

   final static String APPEND = "append";

   private ConsoleViewOwner()
   {
   }

   ConsoleViewOwner(TextView v)
   {
      textView = v;

      v.setOwner(this);
      v.setFilter(this);
   }

   public void performCommand(String c, Object o)
   {
      if (APPEND == c) {
         Range r = textView.selectedRange();
         if (r.length > 0) {
            r = new Range(r.index + r.length, 0);
            textView.selectRange(r);
         }

         int   length = textView.length();
         textView.replaceRangeWithString(r, (String) o);

         r.index += textView.length() - length;
         textView.selectRange(r);
         textView.scrollRangeToVisible(r);

         return;
      }
   }

   // TextViewOwner implementation
   public void attributesDidChange(TextView v, Range r)
   {
   }

   public void attributesWillChange(TextView v, Range r)
   {
   }

   public void linkWasSelected(TextView v, Range r, String urlString)
   {
   }

   public void selectionDidChange(TextView v)
   {
      selectionAtEnd = false;
   }

   public void textWillChange(TextView v, Range r)
   {
   }

   public void textDidChange(TextView v, Range r)
   {
      if (justSawReturn) {
         justSawReturn = false;

         Range previous = v.paragraphForIndex((r.index > 0) ? r.index - 1 : 0);
         String s = textView.stringForRange(previous);

         try {
            writePipe.write(s.toCharArray());
            writePipe.write('\n');
            writePipe.flush();
         } catch (IOException e) {
            e.printStackTrace();
         }
      }
   }

   public void textEditingDidBegin(TextView v)
   {
   }

   public void textEditingDidEnd(TextView v)
   {
   }

   // TextFilter implementation
   public boolean acceptsEvent(Object t, KeyEvent event, Vector keyEvents)
   {
/*
      try {
         char c = event.isExtendedKeyEvent() ? event.keyChar() : (char) event.key;

         writePipe.write(c);
         writePipe.flush();
         // System.out.print("event " + c);
      } catch (IOException e) {
         System.out.println("event err");
      }
*/
      if (event.isReturnKey()) {
         justSawReturn = true;
      }

      return true;
   }
}


class ConsoleReader implements Runnable
{
   ConsoleViewOwner  owner;
   Application       app;

   char[]   inBuf = new char[512];

   private ConsoleReader()
   {
   }

   ConsoleReader(ConsoleViewOwner o)
   {
      owner = o;
      app = Application.application();
   }

   public void run()
   {
      boolean     done = false;
      PipedReader readPipe = owner.readPipe;
      TextView    textView = owner.textView;
      boolean     skipLF = false;

      try {
         while (!done) {
            try {
               int n = readPipe.read(inBuf, 0, inBuf.length);

               done = n < 0;

               // Replace CR and CR-NL with NL.
               int pos = 0;

               for (int i = 0; i < n; ++i) {
                  if (inBuf[i] == '\r') {
                     inBuf[pos++] = '\n';
                     if (i >= (n - 1)) {
                        skipLF = true;
                        break;
                     }

                     if (inBuf[i + 1] == '\n') {
                        i += 1;
                     }
                  } else {
                     inBuf[pos++] = inBuf[i];
                  }
               }

               n = pos;
               pos = 0;

               if (skipLF) {
                  skipLF = false;
                  if (inBuf[0] == '\n') {
                     pos = 1;
                     n -= 1;
                  }
               }

               if (n > 0) {
                  String s = new String(inBuf, pos, n);
                  app.performCommandLater(owner, ConsoleViewOwner.APPEND, s);
               }
            } catch (IOException e) {
               e.printStackTrace();
            }
         }
      } finally {
         System.err.println("reader quit");
      }
   }
}

class ConsoleWriter implements Runnable
{
   ConsoleViewOwner owner;

   private ConsoleWriter()
   {
   }

   ConsoleWriter(ConsoleViewOwner o)
   {
      owner = o;
   }

   public void run()
   {
      CharQueue   charQueue = owner.charQueue;
      PipedWriter writePipe = owner.writePipe;

      while (true) {
         int c = charQueue.get();

         if (c < 0)
            break;

         try {
            writePipe.write((char) c);
         } catch (IOException e) {
            break;
         }
      }
   }
}

final class CharQueue
{
   char[]      characters = new char[1 << 7];
   final int   mask = characters.length - 1;
   int      head = 0;
   int      tail = 0;

   CharQueue()
   {
   }

   synchronized void put(char c)
   {
      int newTail = (tail + 1) % mask;

      if (newTail != head) {
         characters[tail] = c;
         tail = newTail;
      }

      notify();
   }

   synchronized int get()
   {
      try {
         while (head == tail) {
            wait();
         }

         int c = characters[head];

         head = (head + 1) % mask;

         return c;
      } catch (InterruptedException e) {
         return -1;
      }
   }
}
