package to.mambo;

import netscape.application.Application;
import netscape.application.TargetChain;
import netscape.application.ExtendedTarget;


import netscape.application.View;
import netscape.application.TextView;
import netscape.application.TextField;
import netscape.application.ScrollGroup;

import netscape.application.Size;

import netscape.application.Window;
import netscape.application.WindowOwner;
import netscape.application.InternalWindow;
import netscape.application.ExternalWindow;
import netscape.application.Menu;
import netscape.application.MenuItem;

import netscape.constructor.Plan;

import netscape.util.Vector;

import java.io.IOException;

import to.mambo.Console;

import kawa.Shell;
import kawa.lang.Environment;
import kawa.lang.InPort;
import kawa.lang.OutPort;
import kawa.lang.Future;
import kawa.lang.Procedure0;
import kawa.standard.Scheme;

/**
  * A mini application that load plan files.
  */
public class Desktop extends Application
   implements ExtendedTarget, WindowOwner
{
   static final String TEXT_VIEW    = "textView";

   static final String STOP_RUNNING = "stopRunning";

   static final String FILE_NEW = "new";
   static final String FILE_OPEN = "open";
   static final String FILE_SAVE = "save";
   static final String FILE_SAVE_AS = "saveAs";
   static final String FILE_CLOSE = "close";

   static final String EDIT_DELETE = "delete";

   static final String NEW_INTERNAL = "newInternal";
   static final String NEW_EXTERNAL = "newExternal";

   static final String OPTION_EXTERNAL = "optionExternal";

   MenuItem externalItem;

   int consoleNumber;

   public void init()
   {
      super.init();

      Plan mainPlan = Plan.createPlan("main.planb", this);

      ExternalWindow window = mainPlan.externalWindowWithContents();

      window.rootView().setBuffered(true);
      window.setOwner(this);
      window.setTitle("Mambo Desktop");
      setMainRootView(window.rootView());

      {
         Menu menu = new Menu();

         {
            Menu submenu = menu.addItemWithSubmenu("File").submenu();

            submenu.addItem("New", 'N', FILE_NEW, this);
            submenu.addItem("Open", 'O', FILE_OPEN, this);
            submenu.addItem("Save", 'S', FILE_SAVE, TargetChain.applicationChain());
            submenu.addItem("Save as",  FILE_SAVE_AS, TargetChain.applicationChain());
            submenu.addItem("Close", 'W', FILE_CLOSE, TargetChain.applicationChain());
         }

         {
            Menu submenu = menu.addItemWithSubmenu("Edit").submenu();

            submenu.addItem("Cut", 'X'
               , ExtendedTarget.CUT, TargetChain.applicationChain());
            submenu.addItem("Copy", 'C'
               , ExtendedTarget.COPY, TargetChain.applicationChain());
            submenu.addItem("Paste", 'V'
               , ExtendedTarget.PASTE, TargetChain.applicationChain());
            submenu.addItem("Delete"
               ,  EDIT_DELETE, TargetChain.applicationChain());
         }

         {
            Menu submenu = menu.addItemWithSubmenu("Options").submenu();

            externalItem = submenu.addItem("External", OPTION_EXTERNAL
               , this, true);
            externalItem = submenu.addItem("Fonts"
               , ExtendedTarget.SHOW_FONT_CHOOSER, TargetChain.applicationChain());
            externalItem = submenu.addItem("Colors"
               , ExtendedTarget.SHOW_COLOR_CHOOSER, TargetChain.applicationChain());
         }

         window.setMenu(menu);
      }

      window.show();
      window.sizeBy(0, 30);

      Scheme.makeEnvironment();
   }

   public boolean canPerformCommand(String command)
   {
      return true;
   }

   public void performCommand(String command, Object data)
   {
      System.err.println(command);

      if (STOP_RUNNING.equals(command)) {
         stopRunning();
         return;
      }

      if (FILE_NEW.equals(command)) {
         if (externalItem.state()) {
            newExternal();
         } else {
            newInternal();
         }
         return;
      }

      if (NEW_INTERNAL.equals(command)) {
         newInternal();
         return;
      }

      if (NEW_EXTERNAL.equals(command)) {
         newExternal();
         return;
      }

      if (ExtendedTarget.SHOW_FONT_CHOOSER.equals(command)) {
         mainRootView().showFontChooser();
      }

      if (ExtendedTarget.SHOW_COLOR_CHOOSER.equals(command)) {
         mainRootView().showColorChooser();
      }

//      if (ExtendedTarget.SET_COLOR.equals(command)) {
//         mainRootView().setColor((Color) data);
//      }

      System.err.println("Unknown command!");
   }

   public void newInternal()
   {
      Plan plan = Plan.createPlan("internal.planb", this);
      if (plan != null) {
         InternalWindow window = plan.internalWindowWithContents();
         window.setResizable(true);
         window.setCloseable(true);
         window.setContainsDocument(true);

         window.setTitle("Console " + ++consoleNumber);

         Object v = plan.componentNamed("textView");
         if (v instanceof ScrollGroup) {
            v = ((ScrollGroup) v).contentView();
         }

         SchemeShell shell = new SchemeShell(window, (TextView) v);

         window.show();

         window.rootView().setBuffered(true);

         Thread thread = new Future(shell);

         thread.start();
      }
   }

   public void newExternal()
   {
      Plan plan = Plan.createPlan("external.planb", this);
      if (plan != null) {
         ExternalWindow window = plan.externalWindowWithContents();
         window.setContainsDocument(true);
         window.setResizable(true);
         window.setContainsDocument(true);

         window.setTitle("Console " + ++consoleNumber);

         Object v = plan.componentNamed("textView");
         if (v instanceof ScrollGroup) {
            v = ((ScrollGroup) v).contentView();
         }

         SchemeShell shell = new SchemeShell(window, (TextView) v);

         window.show();

         window.rootView().setBuffered(true);

         Thread thread = new Future(shell);

         thread.start();
      }
   }

//  IMPLEMENTATION of WindowOwner INTERFACE

   public void windowDidBecomeMain(Window window)
	{
      if (window instanceof ExternalWindow) {
         setMainRootView(((ExternalWindow) window).rootView());
      }
	}

   public void windowDidHide(Window window)
   {
      performCommandLater(this, STOP_RUNNING, null);
   }

   public void windowDidResignMain(Window window)
   {
   }

   public void windowDidShow(Window window)
   {
   }

   public boolean windowWillHide(Window window)
   {
   	return true;
   }

   public boolean windowWillShow(Window window)
   {
    	return true;
   }

   public void windowWillSizeBy(Window window, Size size)
   {
   }


   public static void main(String args[])
   {
      Main app;

      app = new Main();

      app.run();

      System.exit(0);
   }
}

class SchemeShell extends Procedure0 implements WindowOwner
{
   Window   window;
   Console  console;
   String   name;

   SchemeShell(Window window, TextView view)
   {
      this.window = window;

      window.setOwner(this);

      name = window.title();

      try {
         console = new Console(view);
      } catch (IOException e) {
         e.printStackTrace();
      }

      console.enable();
   }

   public Object apply0()
   {
      try {
         InPort in = new InPort(console.in, name);
         OutPort out = new OutPort(console.out, name);
         out.setErrDefault(out);
/*
         char[] buf = new char[100];
         while (true) {
            int c = in.read();
            if (c < 0)
               break;
            System.err.print("{"+(char)c+"}");
            out.write((char) c);

            int n = in.read(buf);
//            int n = console.in.read(buf);

            System.err.print(" #" + n + " ");

            if (n < 0)
               break;

            for (int i = 0; i < n; ++i) {
               char c = buf[i];
               System.err.print("{"+(char)c+"}");
               out.write((char) c);
            }

            out.flush();
         }
*/

         Shell.run(Environment.getCurrent(), in, out);
      } catch (Exception e) {
         System.err.println("SchemeShell exception:");
         e.printStackTrace();
//      } catch (RuntimeException e) {
//         System.err.println("SchemeShell runtime exception:");
//         e.printStackTrace();
      }

      System.err.println("run done");

      return null;
   }

//  IMPLEMENTATION of WindowOwner INTERFACE

   public void windowDidBecomeMain(Window window)
	{
      if (window instanceof ExternalWindow) {
         Application.application()
            .setMainRootView(((ExternalWindow) window).rootView());
      }
	}

   public void windowDidHide(Window window)
   {
   }

   public void windowDidResignMain(Window window)
   {
   }

   public void windowDidShow(Window window)
   {
   }

   public boolean windowWillHide(Window window)
   {
   	return true;
   }

   public boolean windowWillShow(Window window)
   {
    	return true;
   }

   public void windowWillSizeBy(Window window, Size size)
   {
   }
}
